
# Define a grid command to be sent to the modifyScrollbar proc

set cmd [list grid .xsb -row 1 -column 0 -sticky ew]

# Create the scrollbar. Do not grid. That will be done in
# modifyScrollbar when necessary.

scrollbar .xsb -orient horizontal -command {.c xview}

# Create and grid a canvas using the modifyScrollbar procedure to
# control  the scrollbar 

canvas .c -height 50 -width 50 -scrollregion {0 0 200 50} \
  -xscrollcommand [list modifyScrollbar .xsb $cmd]

grid .c -row 0 -column 0 -sticky nsew

# Configure the column to expand when possible. 

grid columnconfigure . 0 -weight 1

# Put something in the canvas. Just to add interest. 

.c create rectangle 100 20 120 40

################################################################
# proc  modifyScrollbar {scrollbar cmd startFract endFract}--
# Control the  scrollbar appearance.
# Arguments
# scrollbar : The name of the scrollbar  widget
# cmd : The command to display the scrollbar (grid, pack, place)
# startFract: Fraction for the start edge of the slider. PROVIDED BY TCL
# endFract : Fraction for the end edge of the slider. PROVIDED BY TCL
#
#  Results
# If necessary, the scrollbar is displayed in (or removed from) the
# parent frame.
# The slider is modified to reflect new values.

proc modifyScrollbar {scrollbar cmd startFract endFract} {
    if {($endFract < 1.0) ||($startFract > 0)} {
	eval $cmd 
	$scrollbar set $startFract $endFract
    } {
	eval [lindex $cmd 0] forget $scrollbar
    }
}
