package provide scrollFrame 1.1

################################################################
# proc scrollingFrame {name args}--
#    scrollingFrame - Returns the name of a frame within a canvas 
#    attached to vanishing scrollbars.
# Arguments
#   name	The name of the parent frame
#       NOTE :: NON-CONVENTIONAL RETURN - RETURNS THE INTERNAL NAME, NOT
#               THE NAME OF THE PARENT WINDOW!!!
#   args	Arguments to be passed to frame and canvas
# 
# Results
#   Creates 2 frames, a canvas and a two scrollbars.
#   |-------------| <- Outer holding frame
#   | cccccccccc ^| Canvas within outer frame
#   | cffffffffc || Frame within canvas
#   | cf      fc ||
#   | cffffffffc |<-- Vertical Scrollbar within outer frame
#   | cccccccccc V|
#   | <-------->  | Horizontal Scrollbar within outer frame
#   |-------------|
# 
proc scrollingFrame {outerFrame args} {
  if {[string first "." $outerFrame] != 0} {
      error "$outerFrame is not a legitimate window name - must start with `.'"
  }

  # Create the outer frame (or not if it already exists)
  catch {frame $outerFrame}

  # Build the scrollbar commands for the X and Y scrollbar

  # Create and grid the canvas
  set height 200
  set width 300
  set usex 1
  set usey 1
  
  foreach {arg flag} {width -wid* height -hei* usex -usex usey -usey} {
    if {[set p [lsearch $args $flag]] >= 0} {
      set p2 [expr $p +1]
      set $arg [lindex $args $p2]
      set args [lreplace $args $p $p2]
    }
  }

puts "HT: $height WD: $width"

  if {$usey} {
    scrollbar $outerFrame.sby -orient vertical \
        -command "$outerFrame.c yview"
    set cmdy [list modifyScrollBar $outerFrame.sby \
      [list grid $outerFrame.sby -row 0 -column 1 -sticky ns]]
  } else {
    set cmdy ""
  }

  if {$usex} {
    scrollbar $outerFrame.sbx -orient horizontal \
      -command "$outerFrame.c xview" 
    set cmdx [list modifyScrollBar $outerFrame.sbx \
      [list grid $outerFrame.sbx -row 1 -column 0 -sticky ew]]
  } else {
      set cmdx ""
  }
  
  set cvs [canvas $outerFrame.c -height $height -width $width \
      -yscrollcommand $cmdy -xscrollcommand $cmdx]
  grid $outerFrame.c -row 0 -column 0 -sticky news
  
  # Create the scrollbars.  Do not grid. They'll be gridded when 
  #  needed
  # Configure the canvas to expand with its holding frame
  grid rowconfigure    $outerFrame 0 -weight 1
  grid columnconfigure $outerFrame 0 -weight 1

  # Create a frame to go within the canvas.  The various frame
  #  options are applied here.
  eval frame $cvs.f $args
  
  # Place the new frame within the canvas
  $cvs create window 0 0 -window $cvs.f -anchor nw
  
  # Bind frame changes to modify the canvas scrollregion.
  bind $cvs.f <Configure> "$cvs configure -scrollregion \[$cvs bbox all\]"

  return $cvs.f
}



################################################################
#   proc modifyScrollbar {scrollbar cmd startFract endFract}--
#    Control the scrollbar appearance.
# Arguments
#   scrollbar : The name of the scrollbar widget
#   cmd       : The command to display the scrollbar (grid, pack, place)
#   startFract: Fraction for the start edge of the slider. PROVIDED BY TCL
#   endFract  : Fraction for the end edge of the slider.   PROVIDED BY TCL
#
# Results
#   If necessary, the scrollbar is displayed in (or removed from)  the
#   parent frame.
#   The slider is modified to reflect new values.


proc modifyScrollBar {scrollbar cmd startFract endFract} {
#    if {[string first sby $scrollbar] >= 0} {
#    puts "$scrollbar .. $cmd .. $startFract .. $endFract -- [info level] -- [winfo height [winfo parent $scrollbar]] -- [winfo height [winfo parent $scrollbar].c]"
#    }
    if {[info level] > 51} {return}
    if {($startFract > 0) || ($endFract < 1.0)} {
        if {![winfo ismapped $scrollbar]} {
            eval $cmd
	}
        $scrollbar set $startFract $endFract
    } else {
        eval [lindex $cmd 0] forget $scrollbar
    }
}

if {[info exists argv] &&($argv == "-test")} {

  set ff [scrollingFrame .f1 -background pink -height 20 -width 20 ]
  grid .f1 -row 0 -column 0 -sticky news
  
  for {set i 0} {$i < 20} {incr i} {
      label $ff.l_$i -text $i
      grid $ff.l_$i -row $i -column 0
  }

  set ff [scrollingFrame .f2 -background yellow -height 30 -width 30]
  grid .f2 -row 1 -column 0 -sticky news
  
  for {set i 0} {$i < 200} {incr i} {
      label $ff.l_$i -text $i
      grid $ff.l_$i -row [expr $i / 10] -column [expr $i % 10]
  }
  grid rowconfigure . 0 -weight 1
  grid rowconfigure . 1 -weight 3
  grid columnconfigure . 0 -weight 1
  update idle
}
